<?php
class trimax
{
public $version = "2.4 2017-03-12";
public $arraction;

private $db;
private $dbtable;
private $dbtemplates;
private $dbadmins;
private $arrlegalactions;
private $arrgeneralvar;
private $basicdomain;
private $basicmap;
private $basicurl;
private $post;
private $config;

public function startTrimax($config,$post=array())
{
	$this->config=$config;

	# database connection 
	$dbhost=$config["db"]["dbhost"];
	$dbuser=$config["db"]["dbuser"];
	$dbpass=$config["db"]["dbpass"];
	$dbname=$config["db"]["dbname"];
	$dbtable=$config["db"]["dbtable"];
	$dbtemplates=$config["db"]["dbtemplate"];
	$this->dbadmins=$config["db"]["dbadmins"];
	
	$this->dbtable=$dbtable;
	$this->dbtemplates=$dbtemplates;
	$this->db = mysqli_connect($dbhost,$dbuser,$dbpass,$dbname) or die("error connecting to MySQL " . mysqli_error()) ; 
	$query = 'SET NAMES utf8';
	mysqli_query($this->db,$query);
	
	# system basicdomain & basicmap & basicurl
	$protocol = stripos($_SERVER['SERVER_PROTOCOL'],'https') === true ? 'https://' : 'http://';
	$this->basicdomain = $protocol.$_SERVER['HTTP_HOST'];
	$pathroot=$_SERVER['DOCUMENT_ROOT'];
	$basicmap=str_replace("/trimax/trimax.class.php","",__FILE__);
	$this->basicmap=$basicmap . "/";
	$basicurl=str_replace($pathroot,"",$this->basicmap) . "/";
	$this->basicurl=str_replace("//","/",$basicurl);
	
	# set defaults in SESSION and keep this info in $_SESSION for mods and in $config
	$_SESSION["user"]["loginadmin"]=(!isset($_SESSION["user"]["loginadmin"])) ? false : $_SESSION["user"]["loginadmin"];	
	$_SESSION["user"]["userlevel"]=(!isset($_SESSION["user"]["userlevel"])) ? 0 : $_SESSION["user"]["userlevel"];	
	$_SESSION["user"]["publish"]=(!isset($_SESSION["user"]["publish"])) ? false : $_SESSION["user"]["publish"];	
	$_SESSION["mod"]["basicurl"]=$this->basicurl;
	$_SESSION["mod"]["basicmap"]=$this->basicmap;
	$_SESSION["mod"]["post"]=$post;	

	# general VAR
	$this->arrgeneralvar=$config["generalvar"];
	$this->arrgeneralvar["version"]=$this->version;
	$this->arrgeneralvar["basicmap"]=$this->basicmap;
	$this->arrgeneralvar["basicurl"]=$this->basicurl;
	$this->arrgeneralvar["publedit"]=($_SESSION["user"]["publish"]) ? "publisher &rarr; editor" : "editor &rarr; publisher";

	$this->post=$post;

	# list of legal actions
	$sqlres=$this->read("SELECT distinct(action) as distaction FROM $dbtemplates GROUP BY action");
	$arrlegal=array();
	foreach($sqlres AS $key=>$val)
	{
		$arrlegal[]=$val["distaction"];
	}
	$this->arrlegalactions=$arrlegal;
	
	return;
}

// ==============================[ CREATE ] ==============================
public function create($sqlquery,$sqlvalues)
{
	# use as : $T->create("INSERT INTO table_name",array("column1"=>$value1,"column2"=>$value2));
	# test first word, must be INSERT
	$firstword=substr($sqlquery,0,strpos($sqlquery," "));
	$firstword=strtolower(trim($firstword));	
	if($firstword != "insert")
		exit("ERROR TRImax Model: invalid SQL - only an INSERT is permitted: $sqlquery");
	
	if(empty($sqlvalues))
		exit("ERROR TRImax Model: invalid SQL - INSERT: values cannot be empty");

	# make querystring  (if fieldname starts with # than it is a numeric field, no quotes around field)
	$sqlfld="(";
	$sqlval=" VALUES (";
	foreach($sqlvalues AS $key=>$val)
	{
	  $fieldquote=(substr($key,0,1)=="#") ? "" : "'";
	  $fieldname=(substr($key,0,1)=="#") ? substr($key,1) : $key;
		$sqlfld.=$fieldname.",";
		$sqlval.=$fieldquote . addslashes($val) ."$fieldquote" . ",";	
	}
	$sqlfld=substr($sqlfld,0,-1) . ")";
	$sqlval=substr($sqlval,0,-1) . ")";
	$sqlquery="$sqlquery $sqlfld $sqlval";
	mysqli_query($this->db,$sqlquery);
	$newid = mysqli_insert_id($this->db);
	return $newid;	
}

// ==============================[ READ ] ==============================
public function read($sqlquery,$sqlwhere="")
{
	# test first word, must be SELECT
	$firstword=substr($sqlquery,0,strpos($sqlquery," "));
	$firstword=strtolower(trim($firstword));	
	if($firstword != "select")
		exit("ERROR TRImax Model: invalid SQL - only a SELECT is permitted: $sqlquery");
	
	if(!empty($sqlwhere))
	{
		# make array if only 1 where argument	
		$arrwhere=(is_array($sqlwhere)) ? $sqlwhere : array($sqlwhere);
		
		# compare number of placeholders {{n}} with number of where arguments, must be equal
		$nrplaceholders=substr_count($sqlquery,"{{");
		$nrwhereargs=count($arrwhere);
		if($nrplaceholders != $nrwhereargs)
			exit("ERROR TRImax Model: invalid SQL - incorrect number of {{placeholders}} for where clause : $sqlquery");
	
		# 1. test if placeholders {{1}} ... {{n}} are correct
		# 2. test ';' security against SQL injection
		# 3. replace where arguments in SQL
		$argcount=1;
		reset($arrwhere);
		while(strpos($sqlquery,"{{"))
		{
			# 1.
			if(!strpos($sqlquery,"{{$argcount}}"))
				exit("ERROR TRImax Model: invalid SQL - incorrect sequence of {{placeholders}} in where clause : $sqlquery");

			# 2.
			$wherearg=current($arrwhere);			
			if(strpos($wherearg,";"))
				exit("ERROR TRImax: unauthorized SQL-READ - WHERE parameter contains dangerous code for SQL injection: $sqlquery :: $sqlwhere");
			
			# 3.
			$sqlquery=str_replace("{{" . $argcount . "}}",$wherearg,$sqlquery);
			$argcount++;
			
			next($arrwhere);
		}
	}
	
	$queryresult = mysqli_query($this->db,$sqlquery);
	$row=array();		
	while ($currentrow = $queryresult->fetch_assoc()) 
 		$row[]=$currentrow;

	return $row;
}

// ==============================[ UPDATE ] ==============================
public function update($sqlquery,$sqlvalues,$sqlwhere)
{
	# use as : 	$idpag=$T->update("UPDATE $dbtable {{1}} WHERE id={{2}}",array("notitie"=>$newnotitie),$dbid);
	# test first word, must be UPDATE
	# {{1}} moet altijd een array zijn met fields en values
	# {{2,3,4 ...}} zijn argumenten in WHERE

	$firstword=substr($sqlquery,0,strpos($sqlquery," "));
	$firstword=strtolower(trim($firstword));	
	if($firstword != "update")
		exit("ERROR TRImax Model: invalid SQL - only an UPDATE is permitted: $sqlquery");
	
	if(empty($sqlvalues))
		exit("ERROR TRImax Model: invalid SQL - UPDATE: field and values cannot be empty");

	if(empty($sqlwhere))
		exit("ERROR TRImax Model: invalid SQL - UPDATE: empty condition in WHERE is not permitted");

	# make querystring with field & values
	$fieldvalues="SET ";
	foreach($sqlvalues AS $key=>$val)
	{
	  $fieldquote=(substr($key,0,1)=="#") ? "" : "'";
	  $fieldname=(substr($key,0,1)=="#") ? substr($key,1) : $key;
		$fieldvalues.="$fieldname=$fieldquote" . addslashes($val) . "$fieldquote,";
	}
	$fieldvalues=substr($fieldvalues,0,-1);
	$sqlquery=str_replace("{{1}}",$fieldvalues,$sqlquery);
	
	# add WHERE
	# 1. test if placeholders {{2}} ... {{n}} are correct
	# 2. test ';' security against SQL injection
	# 3. replace where arguments in SQL

	# make array if only 1 where argument	
	$arrwhere=(is_array($sqlwhere)) ? $sqlwhere : array($sqlwhere);

	$argcount=2;
	reset($arrwhere);
	while(strpos($sqlquery,"{{"))
	{
		# 1.
		if(!strpos($sqlquery,"{{$argcount}}"))
			exit("ERROR TRImax Model: invalid SQL - UPDATE : incorrect sequence of {{placeholders}} in where clause : $sqlquery");

		# 2.
		$wherearg=current($arrwhere);
		if(strpos($wherearg,";"))
				exit("ERROR TRImax: unauthorized SQL-UPDATE - WHERE parameter contains dangerous code for SQL injection: $sqlquery :: $sqlwhere");
		
		# 3.
		$sqlquery=str_replace("{{" . $argcount . "}}",$wherearg,$sqlquery);
		$argcount++;
		
		next($arrwhere);
	}
	
	$idpag=mysqli_query($this->db,$sqlquery);
		
	return $idpag;
}

// ==============================[ DELETE ] ==============================
public function delete($sqlquery,$sqlwhere)
{
	# use as : 	$T->delete("DELETE FROM $dbtable WHERE id={{1}}",$dbid);
	# test first word, must be DELETE
	# {{1,2,3,4 ...}} zijn argumenten in WHERE
	$firstword=substr($sqlquery,0,strpos($sqlquery," "));
	$firstword=strtolower(trim($firstword));	
	if($firstword != "delete")
		exit("ERROR TRImax Model: invalid SQL - only an DELETE is permitted: $sqlquery");

	if(empty($sqlwhere))
		exit("ERROR TRImax Model: invalid SQL - DELETE: empty condition in WHERE is not permitted");

	# add WHERE
	# 1. test if placeholders {{1}} ... {{n}} are correct
	# 2. test ';' security against SQL injection
	# 3. replace where arguments in SQL

	# make array if only 1 where argument	
	$arrwhere=(is_array($sqlwhere)) ? $sqlwhere : array($sqlwhere);

	$argcount=1;
	reset($arrwhere);
	while(strpos($sqlquery,"{{"))
	{
		# 1.
		if(!strpos($sqlquery,"{{$argcount}}"))
			exit("ERROR TRImax Model: invalid SQL - UPDATE : incorrect sequence of {{placeholders}} in where clause : $sqlquery");

		# 2.
		$wherearg=current($arrwhere);
		if(strpos($wherearg,";"))
				exit("ERROR TRImax: unauthorized SQL-DELETE - WHERE parameter contains dangerous code for SQL injection: $sqlquery :: $sqlwhere");
		
		# 3.
		$sqlquery=str_replace("{{" . $argcount . "}}",$wherearg,$sqlquery);
		$argcount++;
		
		next($arrwhere);
	}
	
	$idpag=mysqli_query($this->db,$sqlquery);
		
	return $idpag;
}

// ====================[ doAction ]==================== // 

public function doAction()
{
	$this->getAction();
	$action=$this->arraction[0];
	

	
 # SYSACTIONS
  switch ($action)
  {
    case "system":
      $arrcontentvars=$this->system($this->arraction);  
			$page=$this->getPage("system",$arrcontentvars);
			return $page;      
      break;

		default:
		return $this->showPage($action);
			
  }	
}	

// ====================[ getAction ]==================== // 

public function getAction()
{
  # redirect to login if needed
  /*
  if(!$this->UserLoggedin())
  	header("location: ./login.php");
	*/
	
	# URL
  $orgurlrequest=$_SERVER['REQUEST_URI'];
	$urlrequest = substr($orgurlrequest,strlen($this->basicurl));
 	$urlrequest=($urlrequest) ? $urlrequest : "start";
	$arraction=explode("/",$urlrequest);	
	$action=$arraction[0];

	# is this a legal action?
	if(!in_array($action,$this->arrlegalactions))
		exit("ERROR TRImax illegal action [$action]");
		
  $this->arraction=$arraction;
  $_SESSION["mod"]["arraction"]=$arraction;	

	return;
}

// ====================[ showPage ]==================== // 

public function showPage($action)
{
	$templatename=$this->getTemplatename($action);
	$arrcontentvars=$this->getContentblocks($action);
	$page=$this->getPage($templatename,$arrcontentvars);
	return $page;
}

// ====================[ getTemplatename ]==================== // 

public function getTemplatename($action)
{
	$dbtemplates=$this->dbtemplates;
	$sqlres=$this->read("SELECT * FROM $dbtemplates WHERE action='{{1}}'",$action);
	if(count($sqlres) != 1)
		exit("ERROR TRImax template not found for $action");
	
	$templatename=$sqlres[0]["template"];
	return $templatename;
}	

// ====================[ getPage ]==================== // 

public function getPage($templatename,$arrcontentvars)
{
	$templatepage=$this->basicmap . "tpl-$templatename" . ".htm";
	if(!file_exists($templatepage))
			exit("ERROR TRImax: template tpl-$templatepage does not exist");

	$page=file_get_contents($templatepage);
	
	# replace all includes [[% _template.tpl %]]
	$localcounter=0;
	while(strpos($page,"[[% ") !== false)
	{
		$posstart=strpos($page,"[[% ");
		$posend=strpos($page," %]]");
		if($posend==0)
			exit("ERROR TRImax: content page has invalid include blocks $posstart-$posend");
		
		$includefile=trim(substr($page,$posstart+4,$posend-$posstart-4));
		$includefullfile=$this->basicmap . $includefile;
		$includecontent=file_get_contents($includefullfile);
		$page=str_replace("[[% $includefile %]]",$includecontent,$page);

		$localcounter++;
		if($localcounter>20)		# max 20 contentblocks
			exit("ERROR TRImax View: content page has an error in include blocks");
	}

	# replace all content {{% contentvar %}}
	$localcounter=0;	
	while(strpos($page,"{{% ") !== false)
	{
		$posstart=strpos($page,"{{% ");
		$posend=strpos($page," %}}");
		if($posstart==0 OR $posend==0)
			exit("ERROR TRImax View: template page $templatename has invalid content variables");
		
		$tvarname=substr($page,$posstart+4,$posend-$posstart-4);	
		$tvarvalue=(isset($arrcontentvars[$tvarname])) ? $arrcontentvars[$tvarname] : "";
		$tvarvalue=stripslashes($tvarvalue);
		$page=str_replace("{{" . "% " .  $tvarname . " %" . "}}",$tvarvalue,$page);

		$localcounter++;
		if($localcounter>20)		# max 20 template content titles
			exit("ERROR TRImax View: template page has invalid content variable structure");
	}
  
  # replace all mods !!% contentvar %!!
	$localcounter=0;	
	while(strpos($page,"!!% ") !== false)
	{
		$posstart=strpos($page,"!!% ");
		$posend=strpos($page," %!!");
		if($posstart==0 OR $posend==0)
			exit("ERROR TRImax View: template page $templatename has invalid mod placeholder");
		
		$modname=substr($page,$posstart+4,$posend-$posstart-4);	
		
		# SESSION UNLOCK - LOCK for include mod
		session_write_close(); // unlock the file
		$cookieses="PHPSESSID=".session_id();
		$opts = array('http' => array('header'=> 'Cookie: ' . $cookieses ."\r\n"));
		$context = stream_context_create($opts);
	  $modcontent=file_get_contents($this->basicdomain . $this->basicurl.$modname,false,$context);
	  session_start(); // Lock the file
		$page=str_replace("!!" . "% " .  $modname . " %" . "!!",$modcontent,$page);

		$localcounter++;
		if($localcounter>20)		# max 20 template content titles
			exit("ERROR TRImax View: template page has invalid content variable structure");
	}

  # custom post processing %variables%
  foreach($this->arrgeneralvar AS $key=>$val)
  {
  	$page=str_replace("((% $key %))",$val,$page);
  }

	# in case of an update with placeholders, keep placeholders
	# !!% mod-name.php %!! => §§%! mod-name.php !%§§
	# ((% generalvar %)) => §§%( generalvar )%§§
	
		$orgchars=array("§§%!","!%§§","§§%(",")%§§");
		$convchars=array("!!%","%!!","((%","%))");
		$page=str_replace($orgchars,$convchars,$page);
		
	return $page;
}

// ====================[ getContentblocks ]==================== // 

function getContentblocks($action)
{
	$dbtable=$this->dbtable;
	$sqlres=$this->read("SELECT * FROM $dbtable WHERE action='{{1}}'",$action);	
	$arrcontentvars=array();
	foreach($sqlres AS $key=>$val)
	{
		$idblock=$val["id"];
		$blockname=$val["blockname"];
		$content=$val["content"];
		if((substr($content,0,1)=='='))
		{
			list($orgtemplate,$orgblockname)=explode("::",substr($content,1));		// cut of first char : =
			$content=$this->getContentblock($orgblockname,$orgtemplate);
		}		
		
		$convmarkdown=$val["markdown"];;
	
		if($_SESSION["user"]["publish"])
		{
			if($convmarkdown)
			{
				$content="
				<div class='markdown trimaxeditbox'>" . 
				Markdown($content) . 
				"</div>
				<div class='trimaxeditbottom'><a href='((% basicurl %))system/page-update/$idblock' role='button'><i class='fa fa-pencil-square fa-2x' aria-hidden='true' style='color: orange;'></i></a></div>
				";	
			}
			else
			{
				$content="
				<div class='trimaxeditbox'>" . 
				$content . 
				"</div>
				<div class='trimaxeditbottom' style='text-align: left;'><a href='((% basicurl %))system/page-update/$idblock' role='button'><i class='fa fa-pencil-square fa-2x' aria-hidden='true' style='color: orange;'></i></a></div>
				";	
			}
			
		}
		else if($convmarkdown)
		{
			$content="
			<div class='markdown'>" . 
			Markdown($content) . 
			"</div>
			";	
		}
		
		$arrcontentvars[$blockname]=$content;
	}
	return $arrcontentvars;
}

// ====================[ getContentblock ]==================== // 

public function getContentblock($blockname,$templatename)
{
	$dbtable=$this->dbtable;
	$sqlres=$this->read("SELECT * FROM $dbtable WHERE blockname='{{1}}' AND action='{{2}}'",array($blockname,$templatename));	
	if(count($sqlres) != 1)
		exit("ERROR TRImax content block $blockname not found or wrong blockname");
		
	$content=$sqlres[0]["content"];
	return $content;
}

// ====================[ system ]==================== // 

private function system($arraction)
{
	$adminswitch=0;
	$isadmin=0;	
	if(isset($_SESSION["user"]))
	{
		$adminswitch=$_SESSION["user"]["loginadmin"];
		$isadmin=$_SESSION["user"]["userlevel"]>1;
	}
	
	$basicurl=$this->basicurl;
	$dbtable=$this->dbtable;
	$systemaction=isset($arraction[1]) ? $arraction[1] : "showsystem";
	$id=isset($arraction[2]) ? $arraction[2] : 0;

	switch ($systemaction)
	{
		case "admin";
			# if adminswitch=OFF => then switch ON and show login form
			# if adminswitch=ON  => then show tpl-system.htm
			
			if($isadmin)
			{
				header("location:$basicurl" . "system");
				exit("ERROR TRImax in system::admin1");				
			}

			if($adminswitch)
			{
				$_SESSION["user"]["loginadmin"]=0;
			}
			else
			{
				$_SESSION["user"]["loginadmin"]=1;
			}
			
			header("location:$basicurl" . "start#admin");
			exit("ERROR TRImax in system::admin2");
			break;
			
		case "showsystem":
			if(!$isadmin)
			{
				header("location:$basicurl" . "start#admin");
				exit("ERROR TRImax in system::showsystem");
			}
			break;

			
		case "logoff";
			unset($_SESSION["user"]);
			header("location:$basicurl" . "start#admin");
			exit("ERROR TRImax in system::logoff");
			break;			


		case "page-update":
			if(!$isadmin)
			{
				header("location:$basicurl" . "start#admin");
				exit("ERROR TRImax in system::page-update");
			}
			
			$sql="SELECT * FROM $dbtable WHERE id='{{1}}'";
			$ressql=$this->read($sql,$id);
			$dbcontent=$ressql[0]["content"];
			$action=$ressql[0]["action"];
			$blockname=$ressql[0]["blockname"];
			$formaction="dbupdate";
			$actionblock="<a href='((% basicurl %))$action'>$action :: $blockname</a>";			
			
			# keep placeholders
			$orgchars=array("!!%","%!!","((%","%))");			
			$convchars=array("§§%!","!%§§","§§%(",")%§§");
			$dbcontent=str_replace($orgchars,$convchars,$dbcontent);
			
			$content="
			<div class='right'>keer terug naar $actionblock</div>
			<form role='form' name='formpag' method='post' action='((% basicurl %))system/page-$formaction/$id'>
			<input name='fid' type='hidden' value='$id'>
			<input name='fpage' type='hidden' value='$action'>
			<div class='form-group'>
			<textarea class='form-control' name='fcontent' placeholder='start content' rows='15'>$dbcontent</textarea></p>
			<p class='center'><input name='submit' type='submit' value='bewaar' class='btn btn-success'> <input name='submit' type='submit' value='cancel'  class='btn btn-default'></p>
			</form>
			</div>
			";			

			$arrcontentvars=array(
				"system-functie"=>"update",
				"system-content"=>$content
				);
			return $arrcontentvars;	
			break;
			
		case "page-dbupdate":
			if(!$isadmin)
			{
				header("location:$basicurl" . "start#admin");
				exit("ERROR TRImax in system::page-dbupdate");
			}
			
			$submit=$_POST["submit"];
			$currentpage=$_POST["fpage"];
		
			if($submit != "bewaar")
			{
				header("location:$basicurl$currentpage");
				break;
			}
			
			$id=$_POST["fid"];
			$content=$_POST["fcontent"];	
			$idpag=$this->update("UPDATE $dbtable {{1}} WHERE id={{2}}",array("content"=>$content),$id);
			header("location:$basicurl$currentpage");
			break;			

		case "adminauth";
			$username=$this->post["fusername"];
			$pwd=$this->post["fpwd"];
			$md5password=MD5("$username::$pwd");
			$sql="SELECT * FROM $this->dbadmins WHERE username='{{1}}'";
			$sqlres=$this->read($sql,$username);

			if(count($sqlres)==1)
			{
				$iduser=$sqlres[0]["id"];
				$username=$sqlres[0]["username"];
				$userlfname=$sqlres[0]["userlfname"];
				$userlevel=$sqlres[0]["userlevel"];
				$dbpassword=$sqlres[0]["userpwd"];
			}
			else
			{
				$dbpassword="INVALID-USER";
			}
			
			if($dbpassword==$md5password && $userlevel)		# userlevel=0 means NO ACCESS
			{
				$_SESSION["user"]["username"]=$username;
				$_SESSION["user"]["userlfname"]=$userlfname;			
				$_SESSION["user"]["id"]=$iduser;			
				$_SESSION["user"]["userlevel"]=$userlevel;
				header("location:$basicurl" . "start");
			}
			else 
			{
				header("location:$basicurl" . "start");
			}
			
			exit("ERROR TRImax in adminauth");
			break;
		
		case "publishonoff":
			if(!$isadmin)
			{
				header("location:$basicurl" . "start#admin");
				exit("ERROR TRImax in system::publishonoff");
			}
			
			$action=$arraction[2];

			$_SESSION["user"]["publish"] = !$_SESSION["user"]["publish"];
			$this->arrgeneralvar["publedit"]=($_SESSION["user"]["publish"]) ? "publisher &rarr; editor" : "editor &rarr; publisher";
			header("location:$basicurl$action");
			break;
			
		case "version":
			exit($this->version);
			break;
		
		case "crud":
			if(!$isadmin)
			{
				header("location:$basicurl" . "start#admin");
				exit("ERROR TRImax in system::crud");
			}			
			$arrcontentvars=$this->crud($arraction);
			return $arrcontentvars;	
			break;

		case "uploadimage":
			$arrcontentvars=$this->uploadimage($arraction);
			return $arrcontentvars;	
			break;
			
		default:
			exit("WARNING: this SYSTEM action is not defined yet");
		
	}
	
	return;
}

// ====================[ crud ]==================== // 

private function crud($arraction)
{
	
	$dbtable=isset($arraction[2]) ? $arraction[2] : "";
	$crudaction=isset($arraction[3]) ? $arraction[3] : "read";
	$id=isset($arraction[4]) ? $arraction[4] : 0;
	
	if($crudaction=="read")
	{
	$content="
		<div style='text-align: right; padding-right: 2.5em;'>
			<a href='{$this->basicurl}system/crud/$dbtable/create' style='border-bottom: none;'><i class='fa fa-plus-square-o fa-2x trimaxcolor' aria-hidden='true'></i></a>
		</div>
	";
	}
	else if(in_array($crudaction,array("create","update","delete")))
	{
	$content="
		<div style='text-align: right; padding-right: 2.5em; margin-bottom: 2em;'>
			<a href='{$this->basicurl}system/crud/$dbtable/read' style='border-bottom: none;'><i class='fa fa-list-alt fa-2x trimaxcolor' aria-hidden='true'></i></a>
		</div>
	";
	}

	switch ($crudaction)
	{
		
		# - - - - - - - - - - - READ - - - - - - - - - - #
		
		case "read":
			# CSS styles
			$csstablediv=$this->config["crudcss"]["tablelist"]["divclass"];
			$csstableclass=$this->config["crudcss"]["tablelist"]["tableclass"];

			$systemfunctie="C<span class='crudaction'>R</span>UD - <span class='crudaction'>lijst</span> van $dbtable";
			$_SESSION["CRUD"]=array();		// reset POST-errors

			// make table with fields to list in correct order
			$content.="
			<div class='$csstablediv'>
				<table class='$csstableclass'>
				<thead>
				<tr>
			";			

			$arrtabstruc=$this->crudTablestruc($dbtable,"LIST");
			foreach($arrtabstruc AS $key=>$val)
			{
				$content.="
				<th>" . $val["LABEL"] . "</th>" . PHP_EOL;
			}
			$content.="
				<th class='center' style='width: 3em;'>U</th>
				<th class='center' style='width: 3em;'>D</th>
				</tr>
				</thead>
				<tbody>
			";
			
			// show table list
			$orderby=$this->config["crud"][$dbtable]["=TABLE="]["SORT"];
			$orderby=($orderby) ? $orderby : "id";
			$sqltable="SELECT * FROM $dbtable ORDER BY $orderby";
			$arrtabcontent=$this->read($sqltable);
			foreach($arrtabcontent AS $keyc=>$valc)
			{
				$content.="
				<tr>
				";

				foreach($arrtabstruc AS $keys=>$vals)
				{
					$fieldname=$vals["FIELD"];
					$fieldlabel=$vals["LABEL"];
					$fieldtype=$vals["TYPE"];
					$fieldid=$valc["id"];
					$fieldwidth=$vals["LISTWIDTH"];
					$fieldcontent=$valc[$fieldname];
					
					$tdwidth=($fieldwidth) ? " width='$fieldwidth'" : "";
					
					if(in_array($fieldtype,array("select","radio","checkbox")))
					{
						if($fieldtype=="checkbox" && strpos($fieldcontent,"::"))
						{
							$fieldcontent='<small>(meerdere waarden)</small>';						
						}
						else
						{
							$arroptions=$this->crudGetoptions($dbtable,$fieldname,$fieldtype);
							$fieldcontent=$arroptions[$fieldcontent];
						}
					}
					$content.="
					<td $tdwidth>$fieldcontent</td>
					";
				}
				$content.="
				<td><a href='{$this->basicurl}system/crud/$dbtable/update/$fieldid' style='border-bottom: none;'><i class='fa fa-pencil-square-o fa-2x trimaxcolor' aria-hidden='true'></i></a></td>
				<td><a href='{$this->basicurl}system/crud/$dbtable/delete/$fieldid' style='border-bottom: none;'><i class='fa fa-trash-o fa-2x trimaxcolor' aria-hidden='true'></i></a></td>
				</tr>
				";
			}

			$content.="
			</tbody>
			</table>
		</div>

		";
			break;
			
			# - - - - - - - - - - - CREATE - - - - - - - - - - #
			
			case "create";
				$systemfunctie="<span class='crudaction'>C</span>RUD - <span class='crudaction'>nieuw</span> in $dbtable";
				$id=0;
				
				// CONTINUE 
				
			# - - - - - - - - - - - UPDATE - - - - - - - - - - #
			
			case "update";
			if($crudaction=="update")
			{
				$systemfunctie="CR<span class='crudaction'>U</span>D - <span class='crudaction'>wijzig</span> in $dbtable";
			}
			$formactionurl=$this->basicurl . "system/crud/$dbtable/dbupdate/$id";

			# CSS styles
			$csstablediv=$this->config["crudcss"]["tableform"]["divclass"];
			$csstableclass=$this->config["crudcss"]["tableform"]["tableclass"];

			// make table with fields in FORM in correct order
			$arrtabstruc=$this->crudTablestruc($dbtable,"FORM");
			$content.="
			<form method='post' action='$formactionurl'>

			<div class='$csstablediv'>
					<table class='$csstableclass'>
					<colgroup>
					<col width='20%'></col>
					<col></col>
					</colgroup>
					<tbody>
			";
			
			// show field list
			$sqltable="SELECT * FROM $dbtable WHERE id=$id";
			$arrtabcontent=$this->read($sqltable);
			$globalposterror="";
			
			foreach($arrtabstruc AS $key=>$val)
			{
				$fieldtype=$val["TYPE"];
				$fieldname=$val["FIELD"];
				$fieldlabel=$val["LABEL"];
				$fieldtype=$val["TYPE"];
				$fieldhtml=$val["HTML"];
				if($fieldtype=="submit")
				{
					continue;
				}
				$fieldvalue=($crudaction=="update") ? $arrtabcontent[0][$fieldname] : "";

				$tablefield=$dbtable. "_" . $fieldname;
				if(isset($_SESSION["CRUD"][$tablefield]["VALUE"]))		// return after POST-ERROR
				{
					$fieldvalue=$_SESSION["CRUD"][$tablefield]["VALUE"];
				}
				$htmlformfield=$this->crudFormfield($dbtable,$fieldname,$fieldlabel,$fieldtype,$fieldvalue,$fieldhtml);
				
				# check POST-ERROR
				if(isset($_SESSION["CRUD"][$tablefield]["ERROR"]))
				{
					$posterror="<br><span style='color: orange;'><i class='fa fa-exclamation-circle' aria-hidden='true' style='color: orange'></i> " . $_SESSION["CRUD"][$tablefield]["ERROR"] . "</span>" ;
					$globalposterror="zie foutboodschappen hierboven";
				}
				else
				{
					$posterror="";
				}
				
				# REQUIRED field => &bull
				if($this->config["crud"][$dbtable][$fieldname]["REQUIRED"])
				{
					$fieldlabel="$fieldlabel &bull;";
				}

				$content.="
					<tr>
					<td class='right bold black'>$fieldlabel $posterror</td>
					<td>
					$htmlformfield
					</td>
					</tr>
					";
			}

			$submittext=($crudaction=="update") ? "wijzigingen bewaren" : "nieuwe gegevens bewaren";
			$htmlsubmit=$this->crudFormfield($dbtable,$fieldname,$fieldlabel,$fieldtype,$submittext,$fieldhtml);
			$globalposterror=($globalposterror) ? "<span style='color: orange;'><i class='fa fa-exclamation-circle' aria-hidden='true' style='color: orange'></i> $globalposterror</span>" : "";

			$content.="
					<tr>
					<td>&nbsp;</td>
					<td>
					$htmlsubmit
					$globalposterror
					</td>
					</tr>
					</tbody>
					</table>
    	</div>
    	
    	</form>
			";

			break;
			
			# - - - - - - - - - - - DELETE - - - - - - - - - - #

			case "delete";
			$systemfunctie="CRU<span class='crudaction'>D</span> - <span class='crudaction'>verwijder</span> in $dbtable";
			
			// make table with fields in FORM in correct order
			$arrtabstruc=$this->crudTablestruc($dbtable,"FORM");
			$content.="
			<div class='crudtable'>
				<div class='table-wrapper'>
					<table>
					<colgroup>
					<col width='20%'></col>
					<col></col>
					</colgroup>
					<tbody>
			";
			
			// show field list only with values, not in inputfields
			$sqltable="SELECT * FROM $dbtable WHERE id=$id";
			$arrtabcontent=$this->read($sqltable);
			foreach($arrtabstruc AS $key=>$val)
			{
				$fieldtype=$val["TYPE"];
				if($fieldtype=="submit")
				{
					continue;
				}				
				$fieldname=$val["FIELD"];
				$fieldlabel=$val["LABEL"];
				$fieldvalue=$arrtabcontent[0][$fieldname];
				if($fieldtype=="textarea" && strlen($fieldvalue)>500)
				{
					$fieldvalue=substr($fieldvalue,0,500) . " (...)";
				}

				if(in_array($fieldtype,array("select","radio","checkbox")))
				{
					if($fieldtype=="checkbox" && strpos($fieldcontent,"::"))
					{
						$fieldvalue='<small>(meerdere waarden)</small>';						
					}
					else
					{
						$arroptions=$this->crudGetoptions($dbtable,$fieldname,$fieldtype);
						$fieldvalue=$arroptions[$fieldvalue];
					}
				}

				$content.="
					<tr>
					<td class='right bold black'>$fieldlabel</td>
					<td>$fieldvalue</td>
					</tr>
					";
			}

			$htmlbutton="<a href='{$this->basicurl}system/crud/$dbtable/dbdelete/$id' class='button special'>verwijder definitief</a>";

			$content.="
					<tr>
					<td>&nbsp;</td>
					<td>
					$htmlbutton
					</td>
					</tr>
					</tbody>
					</table>
    		</div>
    	</div>
    	
			";
			
			$content.="			
					</tbody>
					</table>
    		</div>
    	</div>
			";

			break;
			
		# - - - - - - - - - - - DBUPDATE - - - - - - - - - - #
		
		case "dbupdate":

			# cleanup POST to prepare array with fields and values for update
			$arrupdate=array();
			foreach($this->post AS $key=>$val)
			{
				list($uptable,$upfield)=explode("_",$key);
				if(is_array($val))
				{
					$newval=implode("::",$val);
				}
				else
				{
					$newval=$val;
				}
				$arrupdate[$upfield]=$newval;
			}
			
			# check REQUIRED
			$_SESSION["CRUD"]=array();
			$posterror=false;
			foreach($arrupdate AS $upfield=>$fieldvalue)
			{
				# keep new value in SESSION
				$tablefield=$dbtable. "_" . $upfield;
				$_SESSION["CRUD"][$tablefield]["VALUE"]=$fieldvalue;				
				
				# check REQUIRED
				if($this->config["crud"][$uptable][$upfield]["REQUIRED"] && empty($fieldvalue))
				{
					$_SESSION["CRUD"][$tablefield]["ERROR"]="verplicht veld";
					$posterror=true;
				}
			}
			if($posterror)
			{
				header("location:$this->basicurl" . "system/crud/$dbtable/update/$id");
				exit("STOP");
			}
			
			# write in DB
			if($id)
			{
				$idpag=$this->update("UPDATE $dbtable {{1}} WHERE id={{2}}",$arrupdate,$id);
			}
			else
			{
				$id=$this->create("INSERT INTO $dbtable",$arrupdate);
			}

			header("location:$this->basicurl" . "system/crud/$dbtable");
			break;
			
			# - - - - - - - - - - - DBDELETE - - - - - - - - - - #
			
			case "dbdelete":
			$this->delete("DELETE FROM $dbtable WHERE id={{1}}",$id);
			header("location:$this->basicurl" . "system/crud/$dbtable");
			break;
			
		default:
			exit("ERROR undefined CRUD-action");
	}

	$arrcontentvars=array(
		"system-functie"=>$systemfunctie,
		"system-content"=>$content
		);
	return $arrcontentvars;	
	break;			
	
}

// ====================[ crudTablestruc ]==================== // 

private function crudTablestruc($dbtable,$listform)
{
	$metadata=$this->config["crud"][$dbtable];
		
	// make a table with the correct order of fields :: for LIST and FORM
	$arrtablist=array();
	$arrtabform=array();
	
	foreach($metadata AS $key=>$val)
	{
		if($key=="=TABLE=")
		{
			continue;
		}
		
		$fieldlabel=$val["LABEL"];
  	$listorder=$val["LISTORDER"];
  	$listwidth=$val["LISTWIDTH"];
  	$formorder=$val["FORMORDER"];
  	$fieldtype=$val["TYPE"];
  	$fieldhtml=$val["HTML"];

  	if($listorder)
  	{
			$arrtablist[$listorder]["FIELD"]=$key;
			$arrtablist[$listorder]["LABEL"]=$fieldlabel;
			$arrtablist[$listorder]["TYPE"]=$fieldtype;
			$arrtablist[$listorder]["LISTWIDTH"]=$listwidth;
  	}
  	if($formorder)
  	{
			$arrtabform[$formorder]["FIELD"]=$key;
			$arrtabform[$formorder]["LABEL"]=$fieldlabel;
			$arrtabform[$formorder]["TYPE"]=$fieldtype;
			$arrtabform[$formorder]["HTML"]=$fieldhtml;
  	}
	}
	
	if($listform=="LIST")
	{
		return $arrtablist;
	}

	if($listform=="FORM")
	{
		return $arrtabform;
	}
	
}	

// ====================[ crudFormfield ]==================== // 

private function crudFormfield($dbtable,$fieldname,$fieldlabel,$fieldtype,$fieldvalue,$fieldhtml)
{

	switch ($fieldtype)
	{
		case "text":
		case "textarea":
		case "submit":
			$html=str_replace(array("#dbtable","#fieldname","#fieldlabel","#fieldvalue"),array($dbtable,$fieldname,$fieldlabel,$fieldvalue),$fieldhtml);
		break;
		
		# - - - - - - - - - - - SELECT - - - - - - - - - - #
			
		case "select":
			
			$arroptions=$this->crudGetoptions($dbtable,$fieldname,$fieldtype);

			$htmloptions="";
			foreach($arroptions AS $key=>$val)
			{
				$selected=($fieldvalue==$key) ? " SELECTED " : "";
				$htmloptions.="
				<option value='$key' $selected>$val</option>";
			}
			
			$html=str_replace(array("#dbtable","#fieldname","#fieldlabel","#fieldlist"),array($dbtable,$fieldname,$fieldlabel,$htmloptions),$fieldhtml);
			break;
			
		# - - - - - - - - - - - RADIO - - - - - - - - - - #
			
		case "radio":

			$arroptions=$this->crudGetoptions($dbtable,$fieldname,$fieldtype);

			$newhtml="";
			foreach($arroptions AS $key=>$val)
			{
				$checked=($fieldvalue==$key) ? " CHECKED " : "";
				$newhtml.=str_replace(array("#dbtable","#fieldname","#fieldlabel","#fieldvalue","#checked"),array($dbtable,$fieldname,$val,$key,$checked),$fieldhtml);
			}
			$html=$newhtml;
			break;			

		# - - - - - - - - - - - CHECKBOX - - - - - - - - - - #
			
		case "checkbox":

			$arroptions=$this->crudGetoptions($dbtable,$fieldname,$fieldtype);
			$arrvalues=explode("::",$fieldvalue);
			
			$newhtml="";
			foreach($arroptions AS $key=>$val)
			{
				$checked=(in_array($key,$arrvalues)) ? " CHECKED " : "";
				$newhtml.=str_replace(array("#dbtable","#fieldname","#fieldlabel","#fieldvalue","#checked"),array($dbtable,$fieldname,$val,$key,$checked),$fieldhtml);
			}
			$html=$newhtml;
			break;						
		default:
			exit("ERROR TRImax in crudFormfield : illegal field type");

	}
	return $html;
}

// ====================[ crudGetoptions ]==================== // 

private function crudGetoptions($dbtable,$fieldname,$fieldtype)
{
	$options=$this->config["crud"][$dbtable][$fieldname]["FIELDLIST"];
	
	if(substr($options,0,1)=="{")
	{
		$arroptions=json_decode($options,true);
		if(json_last_error())
		{
			exit("ERROR TRImax in crudGetoptions : invalid JSON-string");
		}
	}
	else if(strtoupper(substr($options,0,6))=="SELECT")
	{
		$arrsel=$this->read($options);
		$arroptions=array();
		foreach($arrsel AS $key=>$val)
		{
			$arropt=array_values($val);
			$optkey=$arropt[0];
			$optval=$arropt[1];
			$arroptions[$optkey]=$optval;
		}
	}
	else
	{
		exit("ERROR TRImax in crudGetoptions : invalid SELECT options");
	}

	return $arroptions;
}

// ====================[ uploadimage ]==================== // 

private function uploadimage()
{
	$fototype="";
	$nieuwenaam="";
	$uploadmessage="";
	$maxfilesize=isset($this->config["admin"]["uploadmaxfile"]) ? $this->config["admin"]["uploadmaxfile"] : 1000000 ;
	if($maxfilesize>4000000)
	{
		$uploadmessage="<div class='uploaderror'>WARNING in config: uploadmaxfile mag niet groter zijn dan 4MB</div>";
	}
	
	# - - - - - - - - - - POST - - - - - - - - - - # 
	
	if(isset($this->post["submit"]) && $this->post["submit"]=="upload foto")
	{
		$fototype=$this->post["fototype"];
		$nieuwenaam=$this->post["fnieuwenaam"];
		$overwrite=isset($this->post["foverwrite"]) ? 1 : 0;
		
		# CHECK POST variables
		if(empty($fototype))
		{
			$uploadmessage="<div class='uploaderror'>maak een keuze uit de fototypes</div>";
		}
		else if(empty($nieuwenaam))
		{
			$uploadmessage="<div class='uploaderror'>kies een naam voor de foto op de server (zonder extensie en zonder spaties)</div>";
		}
		else if(strpos($nieuwenaam," "))
		{
			$uploadmessage="<div class='uploaderror'>kies een naam zonder spaties</div>";
		}
		else	
		{
			# CHECK image upload errors
			$configwidth=$this->config["admin"]["uploadimages"][$fototype]["WIDTH"];
			$configheight=$this->config["admin"]["uploadimages"][$fototype]["HEIGHT"];
			$configmap=$this->config["admin"]["uploadimages"][$fototype]["MAP"];
			$configbasket=$this->config["admin"]["uploadbasketmap"];

			$uploaderror=$this->saveimage("fupload",$nieuwenaam,$configwidth,$configheight,$configbasket,$configmap,$overwrite);
			if(substr($uploaderror,0,12)=="::UPLOADOK::")
			{
				$imageurl=substr($uploaderror,12);
				$uploaderror="";
			}
			
			if($uploaderror)
			{
				$uploadmessage="<div class='uploaderror'>$uploaderror</div>";
			}
			else
			{
				$filename=$_FILES['fupload']['name'];
				$filetype=$_FILES['fupload']['type'];
				$filesize=$_FILES['fupload']['size'];
				$filenametmp=$_FILES['fupload']['tmp_name'];
				$sizemb=number_format($filesize/1000000,2) . "MB";
				$uploadmessage="
				<div class='uploadok'>Het bestand [ $filename $filetype ( $sizemb ) ] werd met succes geüpload naar de server.</div>
				<div style='border: 1px solid silver; background-color: #EAEAEA; padding: 1em 1em 1em 3em; margin-bottom: 1em;'>
				<h3>url van de foto</h3><p>$imageurl</p>
				<h3>markdown code voor de foto</h3><p>![image]($imageurl)</p>
				<div style='text-align: right; font-size: 0.8em;'><a href='$imageurl' target='_blank' style='color: silver;'>bekijk de foto</a></div>
				</div>
				";
			}		
		}
	}
	
	# - - - - - - - - - - FORM - - - - - - - - - - # 
	
	$formactionurl=$this->basicurl . "system/uploadimage";
	
	# lijst samenstellen van type en max-width
	$arrfototypes=array_keys($this->config["admin"]["uploadimages"]);
	$htmlselect="
	<div class='form-group'>
		<select name='fototype' class='form-control'>
		<option value='0'>-- kies een type foto --</option>
	";
	foreach($arrfototypes AS $key=>$val)
	{
		$checked=($val==$fototype) ? " SELECTED " : "";
		$htmlselect.="
		<option value='$val' $checked>$val</option>";
	}
	$htmlselect.="
		</select>
	</div>";
	
	$systemfunctie="upload foto";
	
	$content="
	<div class='trimax uploadimage'>
		$uploadmessage
		<form name='formupload' method='post' enctype='multipart/form-data' action='$formactionurl'>
		<input type='hidden' name='MAX_FILE_SIZE' value='$maxfilesize'>

	  <div class='form-group'>
		<input type='file' name='fupload' accept='image/*' class='form-control' autofocus > 
		</div>
		
		$htmlselect

	  <div class='form-group'>
	    <input type='text' name='fnieuwenaam' placeholder='naam na upload - zonder extensie - geen spaties - geef een * in voor een willekeurige naam' class='form-control' value='$nieuwenaam'>
	  </div>  
		
		<div class='checkbox'>
		  <label><input type='checkbox' name='foverwrite' value='1'>overschrijven indien de naam reeds bestaat</label>
		</div>

	  <div class='form-group'>
	    <input type='submit' name='submit' class='btn btn-info' value='upload foto'>
	  </div>  
	  </form>
  </div>
	";

	$arrcontentvars=array(
	"system-functie"=>$systemfunctie,
	"system-content"=>$content
	);
	return $arrcontentvars;
}

# ====================[ saveimage ]==================== #

private function saveimage($fieldname,$newname,$newwidth,$newheight,$basketmap,$uploaddir,$overwrite)
{
	$acceptedFormats = array('gif','png','jpg','jpeg','GIF','PNG','JPG','JPEG');
	$errortypes = array( 
		0=>"",	
		1=>'dit bestand is te groot (ERROR 1 >upload_max_filesize in php.ini)', 
		2=>'dit bestand is te groot (ERROR 2 >MAX_FILE_SIZE in HTML form)', 
		3=>'dit bestand is slechts gedeeltelijk geüpload (ERROR 3)', 
		4=>'er is geen bestand gekozen, probeer opnieuw (ERROR 4)', 
		6=>'de tijdelijke folder voor upload ontbreekt op de server (ERROR 6)', 
		7=>'dit bestand kan wegens te weinig rechten niet op de server weggeschreven worden (ERROR 7)', 
		8=>'wegens een technisch fout is het bestand niet op de server bewaard (ERROR 8 : a PHP extension stopped the file upload',
		11=>'dit type bestand is niet toegelaten (ERROR 11)'
		); 
		
	$uploaderrornr=$_FILES[$fieldname]["error"];
	$uploaderror=$errortypes[$uploaderrornr];
	if($uploaderror)
	{
		return $uploaderror;
	}
	
	if(!in_array(pathinfo($_FILES[$fieldname]["name"], PATHINFO_EXTENSION), $acceptedFormats))
	{
		$filetype=pathinfo($_FILES[$fieldname]["name"], PATHINFO_EXTENSION);
		return "$filetype : $errortypes[11]";
	}
	
	$image2=$_FILES[$fieldname]['tmp_name'];
	$imageext=substr($_FILES["$fieldname"]["name"],-4);   
	$imageext=($imageext=="jpeg") ? "jpg" : $imageext;
	$mtime=microtime();
	$datetoday=date("Y-m-d");
	list($milsec,$secs)=explode(" ",$mtime);
	$milsec=str_replace("0.","",$milsec);

	if($newname=="*")
	{
		$imagenewfilename=$this->basicmap . "$uploaddir/$datetoday-$secs-$milsec".$imageext;
		$imageurl="((% basicurl %))$uploaddir/$datetoday-$secs-$milsec".$imageext;
	}
	else
	{
		$imagenewfilename=$this->basicmap . "$uploaddir/$newname".$imageext;
		$imageurl="((% basicurl %))$uploaddir/" . $newname .$imageext;
	}
	
	# file exist? overwrite?
	if(!$overwrite)
	{
		if(file_exists($imagenewfilename))
		{
			return "er bestaat reeds een bestand met deze naam en dit mag niet overschreven worden";
		}
	}
	
	# resize
  $check = getimagesize($image2);
  $imagewidth=$check[0];
  $imageheight=$check[1];    
  $resized=false;

	# save uploaded file in basket
	$basketfilename=$this->basicmap . "$basketmap/$datetoday-$secs-$milsec".$imageext;
	move_uploaded_file($image2, $basketfilename);    

	# externe webservice http://rsz.io/
	
  if($imagewidth>$newwidth)
  {
    // OUDE VERSIE met memory exhaust problem voor grote foto's // $this->resize_image_smart($image2, $imagenewfilename,$newwidth,$newheight);
		$url="https://rsz.io/" . $_SERVER['HTTP_HOST'] . $this->basicurl . "$basketmap/$datetoday-$secs-$milsec$imageext?width=$newwidth";
		file_put_contents($imagenewfilename, file_get_contents($url));
    $resized=true;
  }
  else if($imageheight>$newheight)
  {
    // OUDE VERSIE met memory exhaust problem voor grote foto's // $this->resize_image_smart($image2, $imagenewfilename,$imagewidth,$newheight);
		$url="https://rsz.io/" . $_SERVER['HTTP_HOST'] . $this->basicurl . "$basketmap/$datetoday-$secs-$milsec$imageext?height=$newheight";
		file_put_contents($imagenewfilename, file_get_contents($url));    
    $resized=true;
  }

	if(!$resized)
	{
  	rename($basketfilename, $imagenewfilename);    
	}   
	
	return "::UPLOADOK::$imageurl";
  
}

# ====================[ resize_image_smart ]==================== #
# VOORLOPIG NIET GEBRUIKT WEGEND memory exhaust error bij grote foto's, vervangen door webservice rsz.io

private function resize_image_smart($file, $newfilename, $dest_width=null, $dest_height=null)
{
  # http://php.net/manual/en/function.imagecopyresampled.php
  $extension = strtolower(strrchr($file, '.'));
  switch($extension)
  {
      case '.jpg':
      case '.jpeg':
          $orig = @imagecreatefromjpeg($file);
          break;
      case '.gif':
          $orig = @imagecreatefromgif($file);
          break;
      case '.png':
          $orig = @imagecreatefrompng($file);
          break;
      default:
          $orig = @imagecreatefromjpeg($file);
          break;
  }

  $orig_width = imagesx($orig);
  $orig_height = imagesy($orig);
  $vertical_offset = 0;
  $horizontal_offset = 0;
  if($dest_width == null)
  {
    if($dest_height == null)
    {
      die('$dest_width and $dest_height cant both be null!');
    }
    // height is locked
    $dest_width = $dest_height * $orig_width / $orig_height;
  } else {
    if($dest_height == null)
    {
      // width is locked
      $dest_height = $dest_width * $orig_height / $orig_width;
    } else {
      // both dimensions are locked
      $vertical_offset = $dest_height - ($orig_height * $dest_width) / $orig_width;
      $horizontal_offset = $dest_width - ($dest_height * $orig_width) / $orig_height;
      if($vertical_offset < 0) $vertical_offset = 0;
      if($horizontal_offset < 0) $horizontal_offset = 0;
    }
  }
  $img = imagecreatetruecolor($dest_width, $dest_height);
  imagesavealpha($img, true);
  imagealphablending($img, false);
  $transparent = imagecolorallocatealpha($img, 255, 255, 255, 127);
  imagefill($img, 0, 0, $transparent);
  imagecopyresampled($img, $orig, round($horizontal_offset / 2),
                                  round($vertical_offset / 2),
                                  0,
                                  0,
                                  round($dest_width - $horizontal_offset),
                                  round($dest_height - $vertical_offset),
                                  $orig_width,
                                  $orig_height);

  imagejpeg($img, $newfilename);                                
  return;
}

# ====================[ getTextbetween ]==================== #
// getTextbetween() returns the substring beginning after firstchar and ending at secondchar
// ex. getTextbetween("{SQL-","-SQL}",$fielddetails)
public function getTextbetween($delimleft,$delimright, $text)
{
	if ( !strpos("@".$text,$delimleft) && !strpos("@".$text,$delimright) )
		return "";
		
	$startpos=strpos($text,$delimleft)+strlen($delimleft);
	$endpos=strpos($text,$delimright,$startpos);
	$retval=substr($text,$startpos,$endpos-$startpos);
	return $retval;
}

# ====================[ debug ]==================== #
public function debug($var,$continue=false)
{
	echo "<pre>";
	print_r($var);
	echo "</pre>";
	if(!$continue)
		exit;

	return;
}


}

?>